@@ -1,7 +1,5 @@
 use std::collections::HashMap;
-use std::fmt::Display;
 use std::iter::FromIterator;
-use std::str::FromStr;
 
 fn default_h_value() -> String {
     "entry".to_string()
@@ -32,6 +30,7 @@ pub enum Type {
     Read,
     Jam,
     Follow,
+    Event,
     Other(String),
 }
 
@@ -45,41 +44,15 @@ impl ToString for Type {
     }
 }
 
-#[derive(thiserror::Error, Debug)]
-pub struct InvalidPostTypeError {
-    value: String,
-    err: serde_json::Error,
-}
-
-impl PartialEq for InvalidPostTypeError {
-    fn eq(&self, other: &Self) -> bool {
-        self.value == other.value
-    }
-}
-
-impl Display for InvalidPostTypeError {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        f.write_fmt(format_args!(
-            "failed to convert {:?} into a known post type: {:#?}",
-            self.value, self.err
-        ))
-    }
-}
-
-impl FromStr for Type {
-    type Err = InvalidPostTypeError;
-
-    fn from_str(s: &str) -> Result<Self, Self::Err> {
-        serde_json::from_value(serde_json::Value::String(s.to_string())).map_err(|e| Self::Err {
-            err: e,
-            value: s.to_string(),
-        })
-    }
-}
-
 impl PartialEq for Type {
     fn eq(&self, other: &Self) -> bool {
-        std::mem::discriminant(self) == std::mem::discriminant(other)
+        self.to_string() == other.to_string()
+    }
+}
+
+impl Default for Type {
+    fn default() -> Type {
+        Type::Note
     }
 }
 
@@ -126,6 +99,14 @@ impl From<&PostType> for Type {
     }
 }
 
+impl TryFrom<String> for Type {
+    type Error = crate::Error;
+
+    fn try_from(value: String) -> Result<Self, Self::Error> {
+        serde_json::from_value::<Self>(serde_json::Value::String(value)).map_err(crate::Error::JSON)
+    }
+}
+
 impl PartialEq for PostType {
     fn eq(&self, other: &Self) -> bool {
         let ltype: Type = self.into();
@@ -202,23 +183,44 @@ pub fn resolve_from_object(item_mf2: &serde_json::Map<String, serde_json::Value>
                 types
             })
             .and_then(|types| {
-                // FIXME: Do discovery for other combinatory post types.
-                // 'photo' => ['photo', 'note']
-                // 'video' => ['photo', 'video', 'note']
-                // 'media' => ['photo', 'video', 'audio', 'note']
+                let default_type = types.first().cloned().or(Some(Type::Note));
                 if types.contains(&Type::RSVP) && types.contains(&Type::Reply) {
                     types
                         .iter()
                         .find(|post_type| *post_type == &Type::RSVP)
                         .cloned()
+                } else if let Some(resulting_type) = combinatory_type(types) {
+                    Some(resulting_type)
                 } else {
-                    types.first().cloned()
+                    default_type
                 }
             })
     })
 }
 
-pub fn dominant_type<Data>(mf2_data: Data, item_url: &url::Url) -> Option<Type>
+fn combinatory_type(types: Vec<Type>) -> Option<Type> {
+    [
+        (Type::Photo, vec![Type::Photo, Type::Note]),
+        (Type::Video, vec![Type::Video, Type::Photo, Type::Note]),
+        (
+            Type::Media,
+            vec![Type::Audio, Type::Video, Type::Photo, Type::Note],
+        ),
+    ]
+    .iter()
+    .find_map(|(combined_type, expected_types)| {
+        if expected_types
+            .iter()
+            .all(|post_type| types.contains(post_type))
+        {
+            Some(combined_type.to_owned())
+        } else {
+            None
+        }
+    })
+}
+
+pub fn dominant_type<Data>(_mf2_data: Data, _item_url: &url::Url) -> Option<Type>
 where
     Data: ToString,
 {
@@ -272,6 +274,28 @@ mod tests {
                 required_properties: Vec::default()
             })
         );
+
+        assert_eq!(
+            serde_json::from_str::<Vec<PostType>>(
+                r#"
+                [{
+                    "name": "Note",
+                    "type": "note"
+                }, "like"]
+                "#
+            )
+            .ok(),
+            Some(vec![
+                PostType::Expanded {
+                    name: "Note".to_string(),
+                    kind: Type::Note,
+                    h: "entry".to_string(),
+                    properties: Vec::default(),
+                    required_properties: Vec::default()
+                },
+                PostType::Simple(Type::Like)
+            ])
+        );
     }
 
     #[test]
@@ -351,10 +375,4 @@ mod tests {
 
         assert_eq!(dominant_type(text, &item_url), Some(Type::Note));
     }
-
-    #[test]
-    fn type_from_str() {
-        assert_eq!(Type::from_str("note"), Ok(Type::Note));
-        assert!(Type::from_str("foobar").is_err());
-    }
 }
