@@ -1,10 +1,9 @@
-use crate::IConfiguration;
 use serde_json::json;
 use std::fmt::{self, Formatter};
 use std::str::FromStr;
 use std::time::Duration;
 
-use anyhow::anyhow;
+use anyhow::{anyhow, Context};
 use oauth2::basic::BasicTokenIntrospectionResponse;
 use oauth2::revocation::StandardRevocableToken;
 use oauth2::{basic::BasicRevocationErrorResponse, RedirectUrl};
@@ -18,6 +17,8 @@ use oauth2::{
     AuthUrl, AuthorizationCode, ClientId, CsrfToken, PkceCodeChallenge, PkceCodeVerifier, TokenUrl,
 };
 
+use crate::client::{Headers, Request, ResponseValue};
+
 type Client = oauth2::Client<
     BasicErrorResponse,
     Token,
@@ -27,13 +28,6 @@ type Client = oauth2::Client<
     BasicRevocationErrorResponse,
 >;
 
-// FIXME: Implement errors around the IndieAuth flow.
-#[derive(thiserror::Error, Debug)]
-pub enum Error {
-    #[error("An IndieAuth operation failed to completeE: {0}")]
-    Other(anyhow::Error),
-}
-
 // This module holds abstractions for interacting with IndieAuth servers,
 // effectively making this a 'client' module.
 
@@ -143,11 +137,12 @@ pub struct CodeResponse {
     profile: Option<Profile>,
 }
 
-#[derive(Default, Debug)]
-pub struct DiscoveryResult {
-    authorization_endpoints: Vec<String>,
-    token_endpoints: Vec<String>,
-}
+// #[derive(Default, Debug)]
+// pub struct DiscoveryResult {
+//     authorization_endpoints: Vec<url::Url>,
+//     token_endpoints: Vec<url::Url>,
+//     ticket_endpoints: Vec<url::Url>
+// }
 
 pub struct AuthenticationRequest {
     client: Box<Client>,
@@ -220,7 +215,7 @@ impl AuthenticationRequest {
             .add_extra_param("me", self.me.clone())
             .add_scope(Scope::new(self.scopes.join(" ")))
             .set_redirect_uri(std::borrow::Cow::Borrowed(&redirect_url));
-        let (url, state) = dbg!(request.url());
+        let (url, state) = request.url();
 
         Ok((url.to_string(), state.secret().clone()))
     }
@@ -271,12 +266,18 @@ impl TokenResponse<BasicTokenType> for Token {
 }
 
 impl CodeVerificationRequest {
-    pub fn redeem(&self) -> anyhow::Result<Token> {
+    pub fn redeem(&self) -> Result<Token, crate::Error> {
         let client = Client::new(
             ClientId::new(self.client_id.clone()),
             None,
-            AuthUrl::new(self.authorization_endpoint.clone())?,
-            Some(TokenUrl::new(self.token_endpoint.clone())?),
+            AuthUrl::new(self.authorization_endpoint.clone())
+                .context("Failed to parse the authorization URL")
+                .map_err(crate::Error::Other)?,
+            Some(
+                TokenUrl::new(self.token_endpoint.clone())
+                    .context("Failed to parse the authorization URL")
+                    .map_err(crate::Error::Other)?,
+            ),
         );
 
         let mut request = client.exchange_code(AuthorizationCode::new(self.code.clone()));
@@ -288,80 +289,71 @@ impl CodeVerificationRequest {
             None => request,
         };
 
+        // FIXME: Replace this error with a local error for a failed verification attempt.
         request
             .request(http_client)
-            .map_err(|e| anyhow!("{:#?}", e))
+            .with_context(|| {
+                format!("Failed to obtain a response from the token endpoint {:?} about the code {:?} for a token.", self.token_endpoint, self.code)
+            })
+            .map_err(crate::Error::Other)
     }
 }
 
+// pub async fn discover(remote_url: &str) -> Option<DiscoveryResult>
+// {
+//     unimplemented!("write logic to find IndieAuth endpoint discovery.");
+// }
+
 /// Confirms that the provided token is valid with the provided endpoint.
-pub async fn verify<C>(cfg: &C, endpoint: &str, access_token: &str) -> Option<Token>
+pub fn verify(endpoint: &str, access_token: &str) -> Result<Token, crate::Error>
 where
-    C: IConfiguration + Send + Sync,
 {
-    let resp = cfg
-        .request(Some("Requesting information about a token".to_owned()))
-        .request(reqwest::Method::GET, endpoint.to_owned())
-        .header("accept", "application/json")
-        .bearer_auth(access_token)
-        .send()
-        .await
-        .ok();
+    let req = Request {
+        url: url::Url::from_str(endpoint).unwrap(),
+        headers: Headers::from_iter([
+            ("accept", "application/json".to_owned().into_bytes()),
+            (
+                "authorization",
+                format!("bearer {}", access_token).into_bytes(),
+            ),
+        ]),
+        body: vec![],
+    };
 
-    match resp
-        .expect("The endpoint didn't provide a valid response.")
-        .error_for_status()
-    {
-        Ok(resp) => resp
-            .text()
-            .await
-            .ok()
-            .map(|json_text| dbg!(serde_json::from_str(&json_text)).ok())
-            .flatten(),
-        Err(_err) => None,
+    let resp = req.send("get")?;
+
+    match resp.into_json::<Token>()? {
+        ResponseValue::Okay(t) => Ok(t),
+        ResponseValue::Failure(f) => Err(crate::Error::Other(anyhow!(f.to_string()))),
     }
 }
 
-pub async fn redeem<'a, C, R>(
-    cfg: &C,
-    endpoint: &str,
-    redeem_payload: RedeemPayload,
-) -> Result<R, crate::Error>
+pub fn redeem<'a, R>(endpoint: &str, redeem_payload: RedeemPayload) -> Result<R, crate::Error>
 where
-    C: IConfiguration + Send + Sync,
-    R: for<'de> serde::Deserialize<'de>,
+    R: serde::de::DeserializeOwned + std::fmt::Debug,
 {
-    let endpoint_resp = cfg
-        .request(Some(
-            "Requesting information about an authorization request".to_owned(),
-        ))
-        .request(reqwest::Method::POST, endpoint.to_owned())
-        .header("accept", "application/json")
-        .header("content-type", "application/json")
-        .body(
-            json!({
-                "grant_type": "authorization_code",
-                "code": redeem_payload.code,
-                "client_id": redeem_payload.client_id,
-                "redirect_uri": redeem_payload.redirect_uri,
-                "code_verifier": redeem_payload.code_verifier
-            })
-            .to_string(),
-        )
-        .send()
-        .await
-        .map_err(|e| crate::Error::Other(anyhow::anyhow!(e)))?;
+    let req = Request {
+        url: url::Url::from_str(endpoint).unwrap(),
+        headers: Headers::from_iter(vec![
+            ("accept", "application/json"),
+            ("content-type", "application/json"),
+        ]),
+        body: json!({
+            "grant_type": "authorization_code",
+            "code": redeem_payload.code,
+            "client_id": redeem_payload.client_id,
+            "redirect_uri": redeem_payload.redirect_uri,
+            "code_verifier": redeem_payload.code_verifier
+        })
+        .to_string()
+        .into_bytes(),
+    };
 
-    let resp = endpoint_resp
-        .error_for_status()
-        .map_err(|e| anyhow::anyhow!(e))?;
-    log::info!("Handling response from endpoint.");
-    let text = resp
-        .text()
-        .await
-        .map_err(|e| crate::Error::Other(anyhow::anyhow!(e)))?;
-
-    serde_json::from_str(text.as_str()).map_err(|e| crate::Error::Other(anyhow::anyhow!(e)))
+    let resp = req.send("post")?;
+    match resp.into_json::<R>()? {
+        ResponseValue::Okay(t) => Ok(t),
+        ResponseValue::Failure(f) => Err(crate::Error::Other(anyhow!(f.to_string()))),
+    }
 }
 
 #[cfg(test)]
@@ -473,8 +465,8 @@ mod tests {
         assert_eq!(profile.url, "https://jacky.wtf/".parse().ok());
     }
 
-    #[tokio::test(flavor = "multi_thread")]
-    async fn verify_confirms_valid_token() {
+    #[test]
+    fn verify_confirms_valid_token() {
         let cfg = test::init();
         let endpoint = format!("{}/endpoint", mockito::server_url());
         let access_token = "access-token";
@@ -490,8 +482,8 @@ mod tests {
             .create();
 
         endpoint_mock.assert();
-        let result = super::verify(&cfg, &endpoint, access_token).await;
-        assert!(result.is_some());
+        let result = super::verify(&endpoint, access_token);
+        assert!(result.is_ok());
 
         let token = result.unwrap();
         assert_eq!(token.me, "https://jacky.wtf/".parse().unwrap());
@@ -499,8 +491,8 @@ mod tests {
         assert_eq!(token.scopes.to_string(), "read profile email".to_owned());
     }
 
-    #[tokio::test(flavor = "multi_thread")]
-    async fn redeem_resolves_responses() {
+    #[test]
+    fn redeem_resolves_responses() {
         let cfg = test::init();
         let endpoint = format!("{}/endpoint", mockito::server_url());
         let body = serde_json::json!({
@@ -530,9 +522,9 @@ mod tests {
         };
 
         let optional_token: Result<super::Token, crate::Error> =
-            super::redeem(&cfg, &endpoint, payload.clone()).await;
+            super::redeem(&endpoint, payload.clone());
         let optional_code: Result<super::CodeResponse, crate::Error> =
-            super::redeem(&cfg, &endpoint, payload.clone()).await;
+            super::redeem(&endpoint, payload.clone());
         endpoint_mock.assert();
 
         assert!(optional_code.is_ok());
