@@ -1,13 +1,10 @@
 use super::{Request, Response, Status};
-use crate::{Error, IConfiguration};
-use anyhow::Result;
+use crate::{client, Error};
+use anyhow::{Context, Result};
 
 /// Resolves the Webmention endpoints for the provided URL.
-pub async fn endpoints_for<C>(cfg: &C, url: &str) -> Result<Vec<String>, Error>
-where
-    C: IConfiguration + Sync + Send + Sized,
-{
-    let urls = crate::link_rel::for_url(cfg, url, "webmention").await;
+pub fn endpoints_for(url: &str) -> Result<Vec<String>, Error> {
+    let urls = crate::link_rel::for_url(url, "webmention")?;
     if urls.is_empty() {
         Err(Error::NoEndpointsFound {
             url: url.to_owned(),
@@ -19,84 +16,64 @@ where
 }
 
 /// Sends a Webmention.
-pub async fn send<C>(cfg: &C, req: &Request) -> Result<Response, Error>
-where
-    C: IConfiguration + Sync + Send + Sized,
-{
-    let response = cfg
-        .request(Some("Sending a Webmention".to_owned()))
-        .request(reqwest::Method::POST, req.endpoint.clone())
-        .header("accept", "text/html,application/json")
-        .header("content-type", "application/x-www-form-urlencoded")
-        .body(serde_qs::to_string(&req).map_err(|e| Error::Other(e.into()))?)
-        .send()
-        .await
-        .map_err(|err| Error::Other(err.into()))?;
+pub fn send(req: &Request) -> Result<Response, Error> {
+    let req = client::Request {
+        headers: client::Headers::from_iter(vec![
+            ("accept", "text/html,application/json"),
+            ("content-type", "application/x-www-form-urlencoded"),
+        ]),
+        body: serde_qs::to_string(&req)
+            .map_err(|e| Error::Other(e.into()))?
+            .into_bytes(),
+        url: url::Url::parse(&req.endpoint)
+            .context("Failed to parse the URL to be sent in a Webmention sending request.")
+            .map_err(crate::Error::Other)?,
+    };
+    let resp = req.send("post")?;
 
-    match response.status() {
-        reqwest::StatusCode::CREATED => Ok(convert_resp_to_resp(response).await),
-        reqwest::StatusCode::OK => Ok(Response {
-            status: Status::Sent,
-            ..convert_resp_to_resp(response).await
-        }),
-        reqwest::StatusCode::ACCEPTED => Ok(Response {
-            body: response.text_with_charset("utf-8").await.ok(),
-            location: None,
-            status: Status::Accepted,
-        }),
+    match resp.status_code {
+        204 | 200 | 201 => {
+            let location = client::find_header_by_name(&resp.headers, "location")
+                .first()
+                .cloned();
+            let body = Some(resp.body_as_string()?);
+            let status = match resp.status_code {
+                _ => Status::Accepted,
+            };
+
+            Ok(super::Response {
+                endpoint: req.url,
+                body,
+                location,
+                status,
+            })
+        }
         code => Err(Error::WebmentionUnsupportedStatusCode {
             status_code: code,
-            url: req.endpoint.clone(),
+            url: req.url.to_string(),
         }),
     }
 }
 
-async fn convert_resp_to_resp(resp: reqwest::Response) -> Response {
-    let location = resp
-        .headers()
-        .get("location")
-        .map(|v| v.to_str().ok().unwrap_or_default().to_owned());
-    let body = resp.text_with_charset("utf-8").await.ok().and_then(|body| {
-        if body.is_empty() {
-            None
-        } else {
-            Some(body)
-        }
-    });
-    Response {
-        body,
-        location,
-        status: Status::Accepted,
-    }
-}
-
 /// Dispatchs a Webmention.
-pub async fn dispatch<C>(
-    cfg: &C,
+pub fn dispatch(
     source_url: &str,
     target_url: &str,
     additional_options: Option<Request>,
-) -> Result<Vec<Response>, Error>
-where
-    C: IConfiguration + Sync + Send + Sized,
-{
+) -> Vec<Result<Response, Error>> {
     // FIXME: Use `try_flatten_stream` here in the future.
-    let endpoints = endpoints_for(cfg, target_url).await?;
-    let mut responses = vec![];
+    endpoints_for(target_url)
+        .unwrap_or_default()
+        .into_iter()
+        .map(|endpoint| {
+            let mut req = additional_options.clone().unwrap_or_default();
+            req.endpoint = endpoint;
+            req.source = source_url.to_owned();
+            req.target = target_url.to_owned();
 
-    for endpoint in endpoints.into_iter() {
-        let options = additional_options.clone().unwrap_or_default();
-        let req = Request {
-            endpoint,
-            source: source_url.to_owned(),
-            target: target_url.to_owned(),
-            ..options
-        };
-
-        responses.push(send(cfg, &req).await?);
-    }
-
-    Ok(responses)
+            send(&req)
+        })
+        .collect()
 }
 
 #[cfg(test)]
@@ -104,9 +81,10 @@ mod tests {
     use super::*;
     use crate::test;
     use mockito::{mock, Matcher};
+    use url::Url;
 
-    #[tokio::test]
-    async fn send_successful_sync() {
+    #[test]
+    fn send_successful_sync() {
         let cfg = test::init();
         let endpoint_mock = mock("POST", "/endpoint")
             .match_header("content-type", "application/x-www-form-urlencoded")
@@ -122,21 +100,21 @@ mod tests {
             vouch: None,
         };
 
-        let result = super::send(&cfg, &request).await;
+        let result = super::send(&request);
         assert_eq!(
-            result.unwrap_or_default(),
+            result.ok(),
             Some(super::Response {
                 body: None,
                 location: None,
-                status: Status::Sent
+                status: Status::Sent,
+                endpoint: request.endpoint.parse().unwrap()
             })
-            .unwrap_or_default()
         );
         endpoint_mock.assert();
     }
 
-    #[tokio::test]
-    async fn send_captures_async_response() {
+    #[test]
+    fn send_captures_async_response() {
         let cfg = test::init();
         let endpoint_mock = mock("POST", "/endpoint-async-response")
             .match_header("content-type", "application/x-www-form-urlencoded")
@@ -151,21 +129,21 @@ mod tests {
             target,
             vouch: None,
         };
-        let result = super::send(&cfg, &request).await;
+        let result = super::send(&request);
         assert_eq!(
-            result.unwrap_or_default(),
+            result.ok(),
             Some(super::Response {
                 body: None,
                 location: None,
+                endpoint: request.endpoint.parse().unwrap(),
                 status: Status::Sent
             })
-            .unwrap_or_default()
         );
         endpoint_mock.assert();
     }
 
-    #[tokio::test]
-    async fn send_captures_resulting_url() {
+    #[test]
+    fn send_captures_resulting_url() {
         let cfg = test::init();
         let expected_location = "https://theresult.com/wow".to_owned();
         let endpoint_mock = mock("POST", "/endpoint-captures-url")
@@ -183,21 +161,21 @@ mod tests {
             vouch: None,
         };
 
-        let result = super::send(&cfg, &request).await;
+        let result = super::send(&request);
         assert_eq!(
-            result.unwrap_or_default(),
+            result.ok(),
             Some(super::Response {
                 body: None,
                 location: Some(expected_location),
+                endpoint: request.endpoint.parse().unwrap(),
                 status: Status::Sent
             })
-            .unwrap_or_default()
         );
         endpoint_mock.assert();
     }
 
-    #[tokio::test]
-    async fn send_fails_if_status_code_not_in_success_values() {
+    #[test]
+    fn send_fails_if_status_code_not_in_success_values() {
         let cfg = test::init();
         let endpoint_mock = mock("POST", "/endpoint-crashes")
             .match_header("content-type", "application/x-www-form-urlencoded")
@@ -212,20 +190,20 @@ mod tests {
             target,
             vouch: None,
         };
-        let result = super::send(&cfg, &request).await;
+        let result = super::send(&request);
         assert!(result.is_err());
         assert_eq!(
-            result.err().unwrap(),
-            Error::WebmentionUnsupportedStatusCode {
-                status_code: reqwest::StatusCode::INTERNAL_SERVER_ERROR,
+            result.err(),
+            Some(Error::WebmentionUnsupportedStatusCode {
+                status_code: 500,
                 url: request.endpoint.clone()
-            }
+            })
         );
         endpoint_mock.assert();
     }
 
-    #[tokio::test]
-    async fn dispatch_sends_webmentions_to_every_endpoint() {
+    #[test]
+    fn dispatch_sends_webmentions_to_every_endpoint() {
         let cfg = test::init();
         let source_url = "https://jacky.wtf/";
         let max_count = 15;
@@ -265,20 +243,21 @@ mod tests {
             .create();
 
         let request = super::Request::default();
-        let result = super::dispatch(&cfg, &source_url, &target_url, Some(request)).await;
+        let endpoint: Url = request.endpoint.clone().parse().unwrap();
+        let result = super::dispatch(&source_url, &target_url, Some(request));
 
         webpage_mock.assert();
         endpoint_mock.assert();
-        assert!(result.is_ok());
         assert_eq!(
-            result.unwrap(),
-            std::iter::repeat(Response {
+            result,
+            std::iter::repeat_with(|| Ok(Response {
                 status: Status::Accepted,
+                endpoint: endpoint.clone(),
                 body: None,
                 location: None
-            })
+            }))
             .take(max_count)
-            .collect::<Vec<Response>>()
+            .collect::<Vec<Result<Response, crate::Error>>>()
         );
     }
 }
